from __future__ import annotations

import uuid

from pyqtgraph import mkPen, GraphicsObject, mkBrush
from numpy import log10

from gui_qt.Qt import QtCore, QtGui
from gui_qt.lib.pg_objects import LogInfiniteLine, PlotItem


__all__ = ['LineObject', 'MultipointObject', 'RectangleObject']


class BaseObject:
    def __init__(self, color=None):
        self.id = str(uuid.uuid4())
        self.color = color

        self.drawing = None

    def __str__(self):
        raise NotImplementedError


class LineObject(BaseObject):
    def __init__(self, pos: float, angle: float, **kwargs):
        super().__init__(**kwargs)

        self.pos = pos
        self.angle = angle

        self.drawing = LogInfiniteLine(pos=self.pos, angle=self.angle, pen=mkPen(color=self.color.rgb()))

    def __str__(self):
        return f'{"x" if self.angle==90 else "y"}={self.pos}'


class MultipointObject(BaseObject):
    def __init__(self, pts: list, closed: bool = False, **kwargs):
        super().__init__(**kwargs)
        x, y = zip(*pts)
        self.closed = closed

        if self.closed:
            x += (x[0],)
            y += (y[0],)
        self._x = x
        self._y = y

        self.drawing = PlotItem(x=self._x, y=self._y, pen=mkPen(color=self.color.rgb()))

    def __str__(self):
        return f'{len(self._y)-int(self.closed)}-pts'


class RectangleObject(GraphicsObject, BaseObject):
    def __init__(self, left: tuple, right: tuple, **kwargs):
        GraphicsObject.__init__(self)
        BaseObject.__init__(self, **kwargs)

        self.p0 = left
        self.delta = [right[0]-self.p0[0], right[1]-self.p0[1]]
        self.log_mode = [False, False]
        self.generatePicture()

    def __str__(self):
        return 'rectangle'

    def generatePicture(self):
        ## pre-computing a QPicture object allows paint() to run much more quickly,
        ## rather than re-drawing the shapes every time.
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setPen(mkPen('w'))
        p.setBrush(mkBrush('r'))
        p.drawRect(QtCore.QRectF(*self.p0, *self.delta))
        p.end()

    def paint(self, p, _):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        ## boundingRect _must_ indicate the entire area that will be drawn on
        ## or else we will get artifacts and possibly crashing.
        ## (in this case, QPicture does all the work of computing the bouning rect for us)
        return QtCore.QRectF(self.picture.boundingRect())

    def setLogMode(self, x_mode, y_mode):
        if self.log_mode == [x_mode, y_mode]:
            return

        if self.log_mode[0] != x_mode:
            if x_mode:
                xmax = log10(self.p0[0]+self.delta[0])
                self.x_disp = log10(self.p0[0])
                self.dx = xmax - self.x0
            else:
                self.x_disp = self.p0[0]
                self.dx = self.delta[0]
        if args[1] and self.log_mode[1] != args[1]:
            ymax = log10(self.y0 + self.dy)
            self.y0 = log10(self.y0)
            self.dy = ymax - self.y0

        self.generatePicture()

        self.log_mode = [x_mode, y_mode]

        self.informViewBoundsChanged()
